{% import '_search.html.twig' as search %}
{% extends 'base.html.twig' %}

{% block title %}{{ 'queue.list.title'|trans }}{% endblock %}

{% block javascripts %}
<script type="application/javascript">
    {% if not is_pro() %}
    function showLoading() { }
    function hideLoading() { }
    {% endif %}

    function addOrUpdateRow(data, origin, port) {
        var row = $('#items tr[data-uuid="' + data.uuid + '"]').first();

        if (!row.length && data.file) {
            // we try to find by filename
            row = $('#items tr[data-file="' + data.file + '"]').first();
        }

        if (!row.length && data.uuid) {
            console.log("creating row");
            row = $('<tr data-port="' + port + '" class="active">' +
                {% if is_pro() %}'<td class="selector"><input type="checkbox" /></td>' +{% endif %}
                '<td><span class="queue_time"></span><br /><span class="fire_time">?</span></td>' +
                '<td>' +
                '<button class="tool push"><i class="fa fa-paper-plane" aria-hidden="true"></i> {{ 'queue.push'|trans }}</button>' +
                '</td>' +
                '<td class="from"></td>' +
                '<td class="arrow">&rarr;</td>' +
                '<td class="to"></td>' +
                '<td class="tools">' +
                {% if is_pro() %}
                '<button class="tool btn-info logs"><i class="fa fa-search"></i> {{ 'queue.search_logs'|trans }}</button> ' +
                {% endif %}
                '<button class="tool btn-danger discard">{{ 'queue.discard'|trans }}</button> ' +
                '</td>');
            $('#items').append(row);
            row.attr('data-uuid', data.uuid);
        }

        if (!row.length) {
            return;
        }

        row.attr('data-delete', false);
        console.log(data);
        if (data.file) row.attr('data-file', data.file);
        if (data.queue_time) row.find('span.queue_time').text(timestampToDate(data.queue_time));
        if (data.last_attempt) row.find('span.last_attempt').text(timestampToDate(data.last_attempt));
        if (data.fire_time) row.find('span.fire_time').text(timestampToDate(data.fire_time));
        if (data.from) row.find('td.from').text(data.from.replace(/<|>/g, ''));
        if (data.from) row.attr('data-from', data.from.replace(/<|>/g, ''));

        if (data.to) row.attr('data-to', data.to[0].replace(/<|>/g, '')); // we need only first for bounce
        if (data.to) row.find('td.to').text(data.to.map(function(t) { return t.replace(/<|>/g, ''); }).join(', '));
    }

    function timestampToDate(input) {
        var date = new Date(input);

        var year = date.getFullYear();
        var month = "0" + (date.getMonth() + 1);
        var day = "0" + date.getDate();
        var hours = "0" + date.getHours();
        var minutes = "0" + date.getMinutes();

        return year + '-' + month.substr(-2) + '-' + day.substr(-2) + ' ' + hours.substr(-2) + ':' + minutes.substr(-2);
    }

    function processInspect (raw) {
        if (raw.temp_fail_queue) {
            raw.temp_fail_queue.forEach(function (data) {
                addOrUpdateRow({file: data.id, fire_time: data.fire_time});
            });
        }
    }

    var reloading = false;
    function reloadQueue () {
        if (reloading) {
            console.log("still reloading");
            return;
        }

        console.log("reloading!");

        indicator(true);
        showLoading();

        reloading = true;

        $('#items tr').attr('data-delete', true);

        $.when(
            $.ajax("../haraka/11380/queue/list?" + Math.random()),
            $.ajax("../haraka/11381/queue/list?" + Math.random())
        ).done(function (r1d, r2d) {
            r1 = r1d[0];
            r2 = r2d[0];

            r1.forEach(function(data) { addOrUpdateRow(data, 'SMTP', 11380); });
            r2.forEach(function(data) { addOrUpdateRow(data, 'SUBMISSION', 11381); });

            if (r1.length === 0 && r2.length === 0) {
                console.log("showing");
                $('table tfoot').show();
                $('#status-message').text('{{ 'queue.no_email'|trans }}');
            } else {
                console.log("hiding");
                $('table tfoot').hide();
            }

            reloadInspect();

            $('#items tr[data-delete=true]').css('background-color', 'yellow');
            $('#items tr[data-delete=true]').fadeOut(500, function(){ $(this).remove(); reCount(); });
            reCount();
        });
    }

    function indicator(working) {
        //$('#indicator').html(working ? '<i class="fa fa-cog fa-spin"></i>' : '<i class="fa fa-refresh"></i>');
        if (working) {
            $('#indicator').html('<i class="fa fa-cog fa-spin"></i> {{ 'queue.refreshing'|trans }}');
        } else {
            $('#indicator').html('<i class="fa fa-refresh"></i> {{ 'queue.refresh'|trans }}');
        }
    }

    function reloadInspect() {
        console.log("reloading inspect");

        return $.when(
            $.ajax("../haraka/11380/queue/inspect?" + Math.random()),
            $.ajax("../haraka/11381/queue/inspect?" + Math.random())
        ).done(function (r1, r2) {
            processInspect(r1[0]);
            processInspect(r2[0]);

            reloading = false;
            indicator(false);
            hideLoading();
        });
    }

    reloadQueue();

    $('#indicator').click(reloadQueue);

    $('#all').click(function(e) {
        var inputs = $(this).parentsUntil('body', 'table').find('input');
        var to = $(this).prop('checked');

        console.log($(this).parentsUntil('body', 'table'));

        inputs.each(function () {
            $(this).prop('checked', to);
        });
    });

    function push(file, port) {
        console.log("pushing " + file);
        runningAjax = $.ajax("../haraka/" + port + "/queue/" + file + "/push?" + Math.random());
        return runningAjax;
    }

    function discard(file, port) {
        console.log("discarding " + file);
        runningAjax = $.ajax("../haraka/" + port + "/queue/" + file + "/discard?" + Math.random());
        return runningAjax;
    }


    $('#queue-box').on('click', 'button.push', function() {
        var tr = $(this).parentsUntil('table', 'tr');
        var file = tr.attr('data-file');
        var port = tr.attr('data-port');
        push(file, port).then(reloadQueue, reloadQueue);
    });

    {% if is_pro() %}
    $('#queue-box').on('click', 'button.logs', function() {
        var tr = $(this).parentsUntil('table', 'tr');
        var uuid = tr.attr('data-uuid').split('.')[0];
        var bfrom = tr.attr('data-from');
        var bto = tr.attr('data-to');
        // if there is no bfrom, we assume bounce and thus we search source
        window.location.href = "{{ path('log') }}?query=" + (!bfrom ? bto : uuid);
    });


    {% endif %}

    $('#queue-box').on('click', 'button.discard', function() {
        if(!confirm('{{ 'sure.question'|trans }}')) return;
        var tr = $(this).parentsUntil('table', 'tr');
        var file = tr.attr('data-file');
        var port = tr.attr('data-port');
        discard(file, port).then(reloadQueue, reloadQueue);
    });

    var selected = [];
    var searching = '';
    function reCount() {
        selected = [];
        $('tr.active[data-file] input:checked').each(function() {
            var tr = $(this).parentsUntil('table', 'tr');
            var file = tr.attr('data-file');
            var port = tr.attr('data-port');
            selected.push({file: file, port: port});
        });
        setLabel();
    }
    function setLabel() {
        if (selected.length === 0) {
            console.log("=== searching " + searching);
            if (searching.match(/^\s*$/) !== null) {
                $('#discard-label').text('{{ 'queue.discard_all'|trans }}');
                $('#push-label').text('{{ 'queue.push_all'|trans }}');
            } else {
                $('#discard-label').text('{{ 'queue.discard_found'|trans }}');
                $('#push-label').text('{{ 'queue.push_found'|trans }}');
            }
            $('#push-selected').hide();
        } else {
            $('#discard-label').text('{{ 'queue.discard_selected'|trans }} (' + selected.length.toLocaleString() + ')');
            $('#push-label').text('{{ 'queue.push_selected'|trans }} (' + selected.length.toLocaleString() + ')');
            {% if is_pro() %}
            $('#push-selected').show();
            {% endif %}
        }
    }
    setLabel();

    $('#discard-selected').click(function(e) {
        if (!confirm("{{ 'sure.question'|trans }}")) return;

        var processes = [];
        if (selected.length > 0) {
            processes = selected.map(function (m) { return discard(m.file, m.port); });
        } else {
            processes = $('tr.active[data-file]').map(function() {
                var file = $(this).attr('data-file');
                var port = $(this).attr('data-port');
                return discard(file, port);
            });
        }
        $.when.apply($, processes).then(reloadQueue, reloadQueue);
    });

    $('#push-selected').click(function(e) {

        var processes = [];
        if (selected.length > 0) {
            processes = selected.map(function (m) { return push(m.file, m.port); });
        } else {
            processes = $('tr.active[data-file]').map(function() {
                var file = $(this).attr('data-file');
                var port = $(this).attr('data-port');
                return push(file, port);
            });
        }

        $.when.apply($, processes).then(reloadQueue);
    });

    $('#queue-box').on('click', 'input[type=checkbox]', function (e) {
        e.stopPropagation();
        reCount();
    });
    $('#queue-box').on('click', 'a, button', function (e) {
        e.stopPropagation();
    });
    $('#queue-box').on('click', 'tbody td', function () {
        var checkbox = $(this).parent().find('input');
        checkbox.prop('checked', !checkbox.prop('checked'));
        reCount();
    });

    setInterval(reloadQueue, 3500);

    $('#search').keyup(function () {
        searching = $('#search').val();
        filter(searching);
    });

    function filter (keyword) {
        showLoading();

        var filter = keyword.toUpperCase();
        var rows = $('table tbody tr');
        for (var i = 0; i < rows.length; i++) {
            var tds = $(rows[i]).find("td");
            var found = false;

            for (var x = 0; x < tds.length; x++) {
                var txtValue = tds[x].textContent || tds[x].innerText;

                if (txtValue.toUpperCase().indexOf(filter) > -1) {
                    found = true;
                    break;
                } else {
                    found = false;
                }
            }

            if (found) {
                $(rows[i]).addClass('active');
                $(rows[i]).removeClass('disabled');
                rows[i].style.display = "table-row";
            } else {
                $(rows[i]).addClass('disabled');
                $(rows[i]).removeClass('active');
                rows[i].style.display = "none";
            }
        }

        reCount();
        hideLoading();
    }
</script>
{% endblock %}

{% block stylesheets %}
<style>
    .tools { text-align: right; }
    .file, .uuid { font-size: 80%; line-height: 80%; vertical-align: middle; }
    .fire_time { color: gray; }
    #result tr th { vertical-align: top; }
    #indicator { font-weight: normal; color: gray; }
    #result tbody td { vertical-align: middle; }
</style>
{% endblock %}

{% block body -%}

    {% for flashMessage in app.session.flashbag.get('notice') %}
        <div class="alert alert-info">
            {{ flashMessage|trans }}
        </div>
    {% endfor %}

    <div id="queue-box">

    {% if is_pro() %}
        <div style="display: flex">
            <button class="btn btn-warning" id="discard-selected"><i class="fa fa-trash"></i> <span id="discard-label">{{ 'queue.discard_all'|trans }}</span></button>
            <button style="display: none;" class="btn btn-default" id="push-selected"><i class="fa fa-paper-plane"></i> <span id="push-label">{{ 'queue.push_all'|trans }}</span></button>

            <div style="flex-grow: 1">
                {{ search.view(query, 'queue.search.placeholder') }}
            </div>
        </div>
    {% else %}
        <button class="btn btn-warning"><i class="fa fa-trash"></i> {{ 'queue.discard_all'|trans }}</button>
    {% endif %}

    <br /><br />

    <div id="result">
    <table>
        <thead>
            <tr>
                {% if is_pro() %}<th style="width: 1rem;"><input type="checkbox" id="all" /></th>{% endif %}
                <th style="width: 15rem;">{{ 'queue.queued_at'|trans }}<br /><span style="color: gray; font-weight: normal">{{ 'queue.next_attempt'|trans }}</span></th>
                <th style="width: 10rem;"></th>
                <th style="width: calc(50% - 13rem);"></th>
                <th style="width: 1rem;"></th>
                <th style="width: calc(50% - 13rem);"></th>
                <th style="text-align: right;"><a id="indicator" href="#"><i class="fa fa-refresh"></i> {{ 'queue.refresh'|trans }}</a></th>
            </tr>
        </thead>
        <tbody id="items">
        </tbody>
        <tfoot>
            {% if is_pro() %}<td></td>{% endif %}
            <td colspan="999">
                <em id="status-message">{{ 'loading'|trans }}</em>
            </td>
        </tfoot>
    </table>
    </div>
    </div>
{% endblock %}
